/*

Copyright 2011 Danilo Gemoli <nylus77@libero.it>

This file is part of Snake_v1.0.

Snake_v1.0 is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Snake_v1.0 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Snake_v1.0.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "collisioni.h"

extern int tras_rig;
extern int tras_col;

Collisioni::Collisioni()
{
}

bool Collisioni::collisione_muro( Area& area, Serpente& serpe ) const
{
	int posizione = area.get_posizione( serpe.get_testa_riga() - tras_rig, serpe.get_testa_colonna() - tras_col );

	if( posizione == 5 + MURO )
		return true;

	return false;
}

bool Collisioni::collisione_cibo( Area& area, Serpente& serpe ) const
{
	int posizione = area.get_posizione( serpe.get_testa_riga() - tras_rig, serpe.get_testa_colonna() - tras_col );

	if( posizione == 5 + MELA || posizione == 5 + KIWI )
		return true;

	return false;
}

bool Collisioni::collisione_teletrasporto( Area& area, Serpente& serpe ) const
{
	int posizione_testa = area.get_posizione( serpe.get_testa_riga() - tras_rig, serpe.get_testa_colonna() - tras_col );
	int posizione_coda = area.get_posizione( serpe.get_coda_riga() - tras_rig, serpe.get_coda_colonna() - tras_col );

	// NEL RARO CASO IN CUI I DUE SERPENTI PRENDANO IL TELETRASPORTO SULLA STESSA RIGA/COLONNA MA DALLA
	// PARTE OPPOSTO ALLORA BISOGNA DETERMINARE IL GAME OVER
	if( posizione_testa == 5 + ( 5 + TELETRASPORTO_MURO ) )
		collisione_serpente( area, serpe );

	if( posizione_testa == 5 + TELETRASPORTO_MURO || posizione_coda == 5 + TELETRASPORTO_MURO )
		return true;

	return false;	
}

bool Collisioni::collisione_serpente( Area& area, Serpente& serpe ) const 
{
	int posizione = area.get_posizione( serpe.get_testa_riga() - tras_rig, serpe.get_testa_colonna() - tras_col );
	if( posizione == 5 + 4 || posizione == 5 + 8 || posizione == 5 + 6 || posizione == 5 + 2 || posizione == 5 + 5 )
		return true;

	return false;
}

void Collisioni::gestisci_muro( Area& area ) const
{
	nodelay( stdscr, false );
	mvprintw( tras_rig + area.get_righe() / 2, tras_col + area.get_colonne() / 2 - 5, "GAME OVER" );
	mvprintw( tras_rig + area.get_righe() / 2 + 1, tras_col + area.get_colonne() / 2 - 10, "Premi invio per uscire" );

	int tasto;
	do
		tasto = getch();
	while( tasto != '\n' );

	nodelay( stdscr, true );
}

const Collisioni& Collisioni::gestisci_teletrasporto_muro( Area& area, Serpente& serpe ) const
{
	int testa_riga = serpe.get_testa_riga();	
	int testa_colonna = serpe.get_testa_colonna();

	// MURO SU
	if( testa_riga - tras_rig == 0 )
	{
		serpe.set_testa_riga( tras_rig + area.get_righe() - 2 );
		attron( COLOR_PAIR( serpe.get_colore() ) );
		mvprintw( serpe.get_testa_riga(), testa_colonna, "@" );
		attroff( COLOR_PAIR( serpe.get_colore() ) );
		area.aggiungi_a_posizione( serpe.get_testa_riga() - tras_rig, testa_colonna - tras_col, 5 );
	}
	// MURO GIU
	else if( testa_riga - tras_rig == area.get_righe() - 1 )
	{
		serpe.set_testa_riga( tras_rig + 1 );
		attron( COLOR_PAIR( serpe.get_colore() ) );	
		mvprintw( serpe.get_testa_riga(), testa_colonna, "@" );
		attroff( COLOR_PAIR( serpe.get_colore() ) );
		area.aggiungi_a_posizione( serpe.get_testa_riga() - tras_rig, testa_colonna - tras_col, 5 );	
	}
	// MURO DI SINITRA
	else if( testa_colonna - tras_col == 0 )
	{
		serpe.set_testa_colonna( tras_col + area.get_colonne() - 2 );
		attron( COLOR_PAIR( serpe.get_colore() ) );
		mvprintw( testa_riga, serpe.get_testa_colonna(), "@" );
		attroff( COLOR_PAIR( 2 ) );
		area.aggiungi_a_posizione( testa_riga - tras_rig, serpe.get_testa_colonna() - tras_col, 5 );
	}
	// MURO DI DESTRA
	else if( testa_colonna - tras_col == area.get_colonne() - 1 )
	{
		serpe.set_testa_colonna( tras_col + 1 );
		attron( COLOR_PAIR( serpe.get_colore() ) );
		mvprintw( testa_riga, serpe.get_testa_colonna(), "@" );
		attroff( COLOR_PAIR( serpe.get_colore() ) );
		area.aggiungi_a_posizione( testa_riga - tras_rig, serpe.get_testa_colonna() - tras_col, 5 );
	}
	
	int coda_riga = serpe.get_coda_riga();
	int coda_colonna = serpe.get_coda_colonna();

	// MURO SU
	if( coda_riga - tras_rig == 0 )
	{
		serpe.set_coda_riga( tras_rig + area.get_righe() - 2 );
		area.set_posizione( coda_riga - tras_rig, coda_colonna - tras_col, TELETRASPORTO_MURO );
		attron( COLOR_PAIR( 5 ) );
		mvprintw( coda_riga, coda_colonna, "+" );
		attroff( COLOR_PAIR( 5 ) );
		// LA CODA E' NELLA NUOVA POSIZIONE, ED IN QUELLA VECCHIA RIPRISTINA VALORE E DISEGNO DELL' AREA
		// IDEM PER LE ALTRE
	}
	// MURO GIU
	else if( coda_riga - tras_rig == area.get_righe() - 1 )
	{
		serpe.set_coda_riga( tras_rig + 1 );
		area.set_posizione( coda_riga - tras_rig, coda_colonna - tras_col, TELETRASPORTO_MURO );
		attron( COLOR_PAIR( 5 ) );
		mvprintw( coda_riga, coda_colonna, "+" );
		attroff( COLOR_PAIR( 5 ) );
	}
	// MURO DI SINITRA
	else if( coda_colonna - tras_col == 0 )
	{
		serpe.set_coda_colonna( tras_col + area.get_colonne() - 2 );
		area.set_posizione( coda_riga - tras_rig, coda_colonna - tras_col, TELETRASPORTO_MURO );
		attron( COLOR_PAIR( 5 ) );
		mvprintw( coda_riga, coda_colonna, "+" );
		attroff( COLOR_PAIR( 5 ) );
	}
	// MURO DI DESTRA
	else if( coda_colonna - tras_col == area.get_colonne() - 1 )
	{
		serpe.set_coda_colonna( tras_col + 1 );
		area.set_posizione( coda_riga - tras_rig, coda_colonna - tras_col, TELETRASPORTO_MURO );
		attron( COLOR_PAIR( 5 ) );
		mvprintw( coda_riga, coda_colonna, "+" );
		attroff( COLOR_PAIR( 5 ) );
	}

	return *this;
}

const Collisioni& Collisioni::gestisci_cibo( Area& area, Serpente& serpe, Punteggio& punteggio, bool cibo_diversificato ) const
{
	int posizione = area.get_posizione( serpe.get_testa_riga() - tras_rig, serpe.get_testa_colonna() - tras_col );
	

	if( posizione == 5 + MELA )
	{
		serpe.mangia();
		serpe.aumenta_lunghezza();

		Mela* m = new Mela;
		punteggio.cibo( static_cast< Cibo* >( m ) );
		delete m;
	}
	else if( posizione == 5 + KIWI )
	{
		serpe.mangia().mangia();
		serpe.aumenta_lunghezza().aumenta_lunghezza();

		Kiwi* k = new Kiwi;
		punteggio.cibo( static_cast< Cibo* >( k ) );
		delete k;
	}

	genera_cibo( area, cibo_diversificato );
	return *this;
}

void Collisioni::gestisci_serpente( Area& area ) const
{
	nodelay( stdscr, false );
	mvprintw( tras_rig + area.get_righe() / 2, tras_col + area.get_colonne() / 2 - 5, "GAME OVER" );
	mvprintw( tras_rig + area.get_righe() / 2 + 1, tras_col + area.get_colonne() / 2 - 10, "Premi invio per uscire" );

	int tasto;
	do
		tasto = getch();
	while( tasto != '\n' );

	nodelay( stdscr, true );
}
