/*

Copyright 2011 Danilo Gemoli <nylus77@libero.it>

This file is part of Snake_v1.0.

Snake_v1.0 is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Snake_v1.0 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Snake_v1.0.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include "serpente.h"

extern int tras_rig;
extern int tras_col;

Serpente::Serpente( int t_riga, int t_colonna, int cibo, int lung, int col, Area& area )
	: testa_riga( t_riga ), testa_colonna( t_colonna ), coda_riga( t_riga ), coda_colonna( t_colonna ),
	  lunghezza( lung ), colore( col ), cibo_da_digerire( cibo )
{
	attron( COLOR_PAIR( colore ) );
	mvprintw( testa_riga, testa_colonna, "@" );
	attroff( COLOR_PAIR( colore ) );
	area.aggiungi_a_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 5 );
}

int Serpente::get_testa_riga() const
{
	return testa_riga;
}

int Serpente::get_testa_colonna() const
{
	return testa_colonna;
}

int Serpente::get_coda_riga() const
{
	return coda_riga;
}

int Serpente::get_coda_colonna() const
{
	return coda_colonna;
}

int Serpente::get_lunghezza() const
{
	return lunghezza;
}

int Serpente::get_colore() const
{
	return colore;
}

Serpente& Serpente::set_testa_riga( int valore )
{
	testa_riga = valore;
	return *this;
}

Serpente& Serpente::set_testa_colonna( int valore )
{
	testa_colonna = valore;
	return *this;
}

Serpente& Serpente::set_coda_riga( int valore )
{
	coda_riga = valore;
	return *this;
}

Serpente& Serpente::set_coda_colonna( int valore )
{
	coda_colonna = valore;
	return *this;
}

Serpente& Serpente::muovi_destra( Area& area )
{
	area.set_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 6 );
	++testa_colonna;

	attron( COLOR_PAIR( colore ) );
	mvprintw( testa_riga, testa_colonna, "@" );
	attroff( COLOR_PAIR( colore ) );
	area.aggiungi_a_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 5 );

	return *this;
}

Serpente& Serpente::muovi_sinistra( Area& area )
{
	area.set_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 4 );
	--testa_colonna;

	attron( COLOR_PAIR( colore ) );
	mvprintw( testa_riga, testa_colonna, "@" );
	attroff( COLOR_PAIR( colore ) );
	area.aggiungi_a_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 5 );

	return *this;
}

Serpente& Serpente::muovi_su( Area& area )
{
	area.set_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 8 );
	--testa_riga;
	
	attron( COLOR_PAIR( colore ) );
	mvprintw( testa_riga, testa_colonna, "@" );
	attroff( COLOR_PAIR( colore ) );
	area.aggiungi_a_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 5 );

	return *this;
}

Serpente& Serpente::muovi_giu( Area& area )
{
	area.set_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 2 );
	++testa_riga;
	
	attron( COLOR_PAIR( colore ) );
	mvprintw( testa_riga, testa_colonna, "@" );
	attroff( COLOR_PAIR( colore ) );
	area.aggiungi_a_posizione( testa_riga - tras_rig, testa_colonna - tras_col, 5 );

	return *this;
}

Serpente& Serpente::cancella_coda( Area& area )
{
	if( cibo_da_digerire == 0 )
	{
		int tmp_riga = coda_riga;
		int tmp_colonna = coda_colonna;

		int posizione = area.get_posizione( coda_riga - tras_rig, coda_colonna - tras_col );

		switch( posizione )
		{
			case 4 :
			{
				--coda_colonna;
				break;
			}

			case 8 :
			{
				--coda_riga;
				break;
			}

			case 6 :
			{
				++coda_colonna;
				break;
			}
			
			case 2 :
			{
				++coda_riga;
				break;
			}
		}	
		
		mvprintw( tmp_riga, tmp_colonna, " " );	
		area.set_posizione( tmp_riga - tras_rig, tmp_colonna - tras_col, 0 );
		move( testa_riga, testa_colonna );
	}
	else
	{
		--cibo_da_digerire;
	}

	return *this;
}

Serpente& Serpente::mangia()
{
	++cibo_da_digerire;
	return *this;
}

Serpente& Serpente::aumenta_lunghezza() 
{
	++lunghezza;
	return *this;
}
